	-- System variables:
local name = "Interface Profiler"
local version = "1.0.0"
local main = CreateFrame("Frame")

	-- Colors:
local cWhite = "|cffFFFFFF"
local cYellow = "|cffFFFF00"
local cRed = "|cffFF0000"
local cGreen = "|cff00FF00"
local cDefault = cWhite

	-- Other:
local prefix = cYellow .. "[" .. name .. "] " .. cDefault
local autoLoad = false
local characterStr = ""

function Value_Text(txt)
	local result = ""

	if (txt == true) then
		result = cGreen .. tostring(txt)
	elseif (txt == false) then
		result = cRed .. tostring(txt)
	else
		result = cYellow .. txt
	end

	result = result .. cDefault
	return result
end

function Init_Database()
	if (LediiData == nil) then
		LediiData = {}
	end
	if (LediiData[name] == nil) then
		LediiData[name] = {}
		LediiData[name]["AutoLoad"] = false
	end
	--print(prefix .. "Database has been loaded.")
end

function Load_Settings()
	Init_Database()
	autoLoad = LediiData[name]["AutoLoad"]
	--print(prefix .. "Settings has been loaded.")
end

function Data_Reset()
	LediiData[name] = nil
	Init_Database()
	print(prefix .. "All data has been reset.")
end

function Toggle_AutoLoad()
	autoLoad = not(autoLoad)
	LediiData[name]["AutoLoad"] = autoLoad
	print(prefix .. "Auto load set to " .. Value_Text(autoLoad) .. ".")
end

function Save_Interface(profileName)
	profileName = profileName or "Default"

	-- Init database
	if (LediiData[name][profileName] == nil) then
		LediiData[name][profileName] = {}
		LediiData[name][profileName]["PlayerFrame"] = {}
		LediiData[name][profileName]["TargetFrame"] = {}
		LediiData[name][profileName]["FocusFrame"] = {}
	end

	-- Store data
	point, relTo, relPoint, offX, offY = PlayerFrame:GetPoint(1)
	LediiData[name][profileName]["PlayerFrame"]["Point"] = point
	LediiData[name][profileName]["PlayerFrame"]["RelativePoint"] = relPoint
	LediiData[name][profileName]["PlayerFrame"]["OffsetX"] = offX
	LediiData[name][profileName]["PlayerFrame"]["OffsetY"] = offY

	point, relTo, relPoint, offX, offY = TargetFrame:GetPoint(1)
	LediiData[name][profileName]["TargetFrame"]["Point"] = point
	LediiData[name][profileName]["TargetFrame"]["RelativePoint"] = relPoint
	LediiData[name][profileName]["TargetFrame"]["OffsetX"] = offX
	LediiData[name][profileName]["TargetFrame"]["OffsetY"] = offY

	point, relTo, relPoint, offX, offY = FocusFrame:GetPoint(1)
	LediiData[name][profileName]["FocusFrame"]["Point"] = point
	LediiData[name][profileName]["FocusFrame"]["RelativePoint"] = relPoint
	LediiData[name][profileName]["FocusFrame"]["OffsetX"] = offX
	LediiData[name][profileName]["FocusFrame"]["OffsetY"] = offY

	LediiData[name][profileName]["Loaded"] = {}
	LediiData[name][profileName]["Loaded"][characterStr] = true
	print(prefix .. "Interface saved to profile " .. Value_Text(profileName) .. ".")
end

function Not_Loaded(profileName)
	profileName = profileName or "Default"
	local notLoaded = false

	if (LediiData[name][profileName]["Loaded"] == nil) then
		LediiData[name][profileName]["Loaded"] = {}
		LediiData[name][profileName]["Loaded"][characterStr] = true
		notLoaded = true
	else
		if (LediiData[name][profileName]["Loaded"][characterStr] == nil) then
			LediiData[name][profileName]["Loaded"][characterStr] = true
			notLoaded = true
		end
	end

	return notLoaded
end

function Load_Interface(profileName)
	profileName = profileName or "Default"

	if (LediiData[name][profileName] ~= nil) then
		if (Not_Loaded(profileName)) then

			point = LediiData[name][profileName]["PlayerFrame"]["Point"]
			relPoint = LediiData[name][profileName]["PlayerFrame"]["RelativePoint"]
			offX = LediiData[name][profileName]["PlayerFrame"]["OffsetX"]
			offY = LediiData[name][profileName]["PlayerFrame"]["OffsetY"]
			PlayerFrame:ClearAllPoints()
			PlayerFrame:SetPoint(point, nil, relPoint, offX, offY)
			PlayerFrame.SetPoint = function() end
			PlayerFrame:SetUserPlaced(true)

			point = LediiData[name][profileName]["TargetFrame"]["Point"]
			relPoint = LediiData[name][profileName]["TargetFrame"]["RelativePoint"]
			offX = LediiData[name][profileName]["TargetFrame"]["OffsetX"]
			offY = LediiData[name][profileName]["TargetFrame"]["OffsetY"]
			TargetFrame:ClearAllPoints()
			TargetFrame:SetPoint(point, nil, relPoint, offX, offY)
			TargetFrame.SetPoint = function() end
			TargetFrame:SetUserPlaced(true)

			point = LediiData[name][profileName]["FocusFrame"]["Point"]
			relPoint = LediiData[name][profileName]["FocusFrame"]["RelativePoint"]
			offX = LediiData[name][profileName]["FocusFrame"]["OffsetX"]
			offY = LediiData[name][profileName]["FocusFrame"]["OffsetY"]
			FocusFrame:ClearAllPoints()
			FocusFrame:SetPoint(point, nil, relPoint, offX, offY)
			FocusFrame.SetPoint = function() end
			FocusFrame:SetUserPlaced(true)

			print(prefix .. "Interface loaded from profile " .. Value_Text(profileName) .. ".")
		else
			--print(prefix .. "Profile " .. Value_Text(profileName) .. " is already loaded.")
		end
	else
		--print(prefix .. "Nothing saved in profile " .. Value_Text(profileName) .. ".")
	end
end

function Load_Ledii_Default()
	local xOff = 400;
	local y1 = 200;
	local x2 = 18;
	local y2 = 300;

	PlayerFrame:ClearAllPoints()
	PlayerFrame:SetPoint("BOTTOM", -xOff, y1)
	PlayerFrame.SetPoint = function() end
	PlayerFrame:SetUserPlaced(true)

	TargetFrame:ClearAllPoints()
	TargetFrame:SetPoint("BOTTOM", xOff, y1)
	TargetFrame.SetPoint = function() end
	TargetFrame:SetUserPlaced(true)

	FocusFrame:ClearAllPoints()
	FocusFrame:SetPoint("BOTTOM", x2, y2)
	FocusFrame.SetPoint = function() end
	FocusFrame:SetUserPlaced(true)

	print(prefix .. "Loaded " .. Value_Text("Ledii's Default") .. " interface.")
end

function Handle_Events(self, event, ...)
	-- Player login:
	if (event == "PLAYER_LOGIN") then
		print(prefix .. "Version " .. version .. " loaded!")
		print(prefix .. "Type " .. Value_Text("/ip help") .. " to see a list of commands.")
		characterStr = GetUnitName("player", true)

		Load_Settings()

		if (autoLoad) then
			Load_Interface()
		end
	end
end

-- Chat commands:
SLASH_IPROFILER1 = "/ip"
function SlashCmdList.IPROFILER(msg, editbox)
	if (msg == "save") then
		Save_Interface()

	elseif (msg == "load") then
		Load_Interface()

	elseif (msg == "ledii") then
		Load_Ledii_Default()

	elseif (msg == "autoload") then
		Toggle_AutoLoad()

	elseif (msg == "clear") then
		Data_Reset()

	elseif (msg == "help") then
		print(prefix .. "List of interface commands:\n"
		.. Value_Text("/ip save") .. ": Saves the interface to the current profile.\n"
		.. Value_Text("/ip load") .. ": Loades the interface from the current profile.\n"
		.. Value_Text("/ip clear") .. ": Clears all stored data. (In case it's needed.)\n"
		.. Value_Text("/ip setprofile") .. ": Work in progress...\n"
		.. Value_Text("/ip ledii") .. ": Loades my personal setup if you would like to try that out.\n"
		.. Value_Text("/ip autoload") .. ": Toggles the autoload function. (Will automaticly load changes for all other characters that use this profile.)\n"
		)
	end
end

-- Events to register:
main:RegisterEvent("PLAYER_LOGIN")
main:SetScript("OnEvent", Handle_Events)